/*
 * FieldValue.cpp
 *
 * Copyright (C) 1999 Stephen F. White
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program (see the file "COPYING" for details); if 
 * not, write to the Free Software Foundation, Inc., 675 Mass Ave, 
 * Cambridge, MA 02139, USA.
 */

#include <stdio.h>
#include "stdafx.h"

#include "FieldValue.h"
#include "SFMFTypes.h"
#include "Scene.h"
#include "Element.h"
#include "DuneApp.h"

FieldValue::FieldValue() {
    _refs = 0;
}

//FieldValue::FieldValue(const FieldValue &fieldValue)
//{
//    _refs = 0;
//}

FieldValue *
FieldValue::addNode(Node * /* node */, int /* index */) const
{
    assert(0);
    return NULL;
}

FieldValue *
FieldValue::removeNode(Node * /* node */) const
{
    assert(0);
    return NULL;
}

int indentf(int f, int indent)
{
    int tabs = indent / 8;
    int spaces = indent % 8;
    char *buf = new char[tabs + spaces + 1];
    char *b = buf;
    int ret;

    for (int i = 0; i < tabs; i++)
        *b++ = '\t';

    for (int j = 0; j < spaces; j++)
        *b++ = ' ';

    *b = '\0';
    ret= mywritestr(f , buf);
    delete[] buf;
    return(ret);
}

const char *
FieldValue::getEcmaScriptIndent(int flags)
{
    if (flags == EL_EVENT_IN) {
        static const char* readIndent  = "      ";
        return readIndent;
    } else {
        static const char* writeIndent = "   ";
        return writeIndent;
    }
}

bool        
FieldValue::checkInput(char *line)
{
    if (getNumbersPerLine() == 0)
        return true;

    int numbersPerLine = 0;

    bool hasSign = false;
    bool hasNumber = false;
    bool hasComma = false;
    bool hasExponent = false;
    bool hasExponentSign = false;
    bool hasExponentComma = false;

    bool whitespace = true;
    for (int i = 0; i < strlen(line); i++) {
        if ((line[i] == ' ') || (line[i] == '\t'))
            whitespace = true;
        else {
            // check valid characters 
            // integer: [+-][0-9]* 
            // float:   [+-][0-9]*\.[0-9]*[eE][+-][0-9]*\.[0-9]*
            if (whitespace == true) {
                whitespace = false;
                numbersPerLine++;
                hasNumber = false;
                hasSign = false;
                hasComma = false;
                hasExponent = false;
                hasExponentSign = false;
                hasExponentComma = false;
            }
            bool validNumber = false;
            if ((line[i] >= '0') && (line[i] <= 9)) {
                validNumber = true;
                hasNumber = true;
            }
            if ((line[i] == '+') && (line[i] == '-')) {
                validNumber = true;
                if (needCheckFloat() && hasExponent)
                    if (hasExponentSign)
                        return false;
                    else
                        hasExponentSign = true;
                else
                    if (hasSign)
                        return false;
                    else
                        hasSign = true;
            }
            if (needCheckFloat() && (line[i] == '.')) {
                validNumber = true;
                if (needCheckFloat() && hasExponent)
                    if (hasExponentComma)
                        return false;
                    else
                        hasExponentComma = true;
                else
                    if (hasComma)
                        return false;
                    else
                        hasComma = true;
                if ((line[i] == 'e') && (line[i] == 'E')) {
                    if (!needCheckFloat())
                        return false;
                    validNumber = true;
                    if (!hasNumber)
                        return false;
                    if (hasExponent)
                        return false;
                    else
                        hasExponent = true;
                }
            if (!validNumber)
                return false; 
            }
        }
    }
    if (numbersPerLine != getNumbersPerLine())
        return false;
    return true;   
}

FieldValue *
rewriteField(FieldValue *value, const char *oldBase, const char *newBase)
{
    MyString r;

    switch (value->getType()) {
      case SFSTRING:
        r = rewriteURL(((SFString *) value)->getValue(), oldBase, newBase);
#ifdef _WIN32
        // handle invalid paths like c:/something
        if (r != NULL)
            r.gsubOnce("|",":"); 
#endif
        return new SFString(r);
      case MFSTRING:
      {
        bool flag = false;
        StringArray *a = new StringArray();
        int n = ((MFString *) value)->getSize();
        if (n != 0)
            flag = !isSortOfEcmascript(((MFString *) value)->getValue(0));
        for (int i = 0; i < n; i++) {
            const char* url = (const char*) ((MFString *) value)->getValue(i);
            if (notURN(url) && flag) {
               /* need to replace "vrmlscript:" ? */
               (*a)[i] = rewriteURL(url, oldBase, newBase);
#ifdef _WIN32
               // handle invalid paths like c:/something
               if ((*a)[i] != NULL)
                   ((*a)[i]).gsubOnce("|",":"); 
#endif
               }
            else
               (*a)[i] = url;
 
        }
        return new MFString(a);
      }
      default:
        assert(0);
        return NULL;
    }
}

int MFieldValue::write(int f, int indent) const
{ 
    int stride = getStride();
    // SFImage is not a real MFType, it may not use brackets
    if ((getType() != SFIMAGE) && (getSFSize() != 1)) {
        if (!TheApp->GetkrFormating()) {
            RET_ONERROR( mywritestr(f, "\n") )
            TheApp->incSelectionLinenumber();
            RET_ONERROR( indentf(f, indent + TheApp->GetIndent()) )
        }
        RET_ONERROR( mywritestr(f, "[\n") )
        TheApp->incSelectionLinenumber();
    }
    RET_ONERROR( writeData(f, indent) )
    // SFImage is not a real MFType, it may not use brackets
    if ((getType() != SFIMAGE) && (getSFSize() != 1)) {
        if (!TheApp->GetkrFormating())
            RET_ONERROR( indentf(f, indent + TheApp->GetIndent()) )
        else
            RET_ONERROR( indentf(f, indent) )
        RET_ONERROR( mywritestr(f, "]\n") )
        TheApp->incSelectionLinenumber();
    }
    return(0);
}

void
MFieldValue::getDiff(IntArray *newIndices, IntArray *deleteIndices,
                     MFieldValue *old)
{
    int lastIndex = 0;
    int i;
    for (i = 0; i < getSFSize(); i++) {
        bool found = false;
        int j;
        // continue search of next item after result of last search 
        for (j = lastIndex + 1; j < old->getSFSize(); j++)
            if (getSFValue(i)->equals(old->getSFValue(j))) {
                found = true;
                lastIndex = j;
                break;
            }
        if (found == false)
            for (j = 0; (j < (lastIndex + 1)) && (j < old->getSFSize()); j++)
                if (getSFValue(i)->equals(old->getSFValue(j))) {
                    found = true;
                    lastIndex = j;
                    break;
                }
        if (found == false)
            newIndices->append(i);
    }
    lastIndex = 0;
    for (i = 0; i < old->getSFSize(); i++) {
        bool found = false;
        int j;
        // continue search of next item after result of last search 
        for (j = lastIndex + 1; j < getSFSize(); j++)
            if (old->getSFValue(i)->equals(getSFValue(j))) {
                found = true;
                lastIndex = j;
                break;
            }
        if (found == false)
            for (j = 0; (j < (lastIndex + 1)) && (j < getSFSize()); j++)
                if (old->getSFValue(i)->equals(getSFValue(j))) {
                    found = true;
                    lastIndex = j;
                    break;
                }
        if (found == false)
            deleteIndices->append(i);
    }
}

int
FieldValue::writeAc3d(int filedes, int indent) const
{
    int appIndent = TheApp->GetIndent();
    TheApp->SetIndent(0);
    RET_ONERROR( writeData(filedes, indent) )
    TheApp->SetIndent(appIndent);
    return 0;
}

int
FieldValue::writeCattGeo(int filedes, int indent) const
{
    int appIndent = TheApp->GetIndent();
    TheApp->SetIndent(0);
    RET_ONERROR( writeData(filedes, indent) )
    TheApp->SetIndent(appIndent);
    return 0;
}


