/*
 * FieldValue.h
 *
 * Copyright (C) 1999 Stephen F. White
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program (see the file "COPYING" for details); if 
 * not, write to the Free Software Foundation, Inc., 675 Mass Ave, 
 * Cambridge, MA 02139, USA.
 */

#ifndef _FIELDVALUE_H
#define _FIELDVALUE_H

#ifndef _ARRAY_H
#include "Array.h"
#endif
#ifndef _DUNE_STRING_H
#include "MyString.h"
#endif
#ifndef _ELEMENT_H
#include "Element.h"
#endif

class Scene;
class Node;

#include "Arrays.h" 

#include "Types.h"
#include "Swap.h"

class FieldValue {
public:
                        FieldValue();
//                      FieldValue(const FieldValue &value);
    virtual            ~FieldValue() {}
    
    // write data in VRML syntax to file
    virtual int         writeData(int filedes, int indent,
                                  bool compactFormat = true) const = 0;
    virtual int         write(int filedes, int indent) const
                            { return writeData(filedes, indent); }
    // write data in C++ syntax to file (not implemented yet)
    //virtual int         writeCC(int filedes, char* variableName) const = 0;
    virtual int         writeAc3d(int filedes, int indent) const;
    virtual int         writeCattGeo(int filedes, int indent) const;

    virtual bool        readLine(int index, char *line) = 0;

    virtual bool        checkInput(char *line);
    virtual int         getNumbersPerLine(void) const = 0;
    virtual bool        needCheckFloat(void) const { return false; }

    virtual int         getType() const = 0;
    virtual const char *getTypeName() const = 0;
    virtual bool        equals(const FieldValue *value) const = 0;
    virtual FieldValue *addNode(Node *node, int index = -1) const;
    virtual FieldValue *removeNode(Node *node) const;

    virtual void        clamp(const FieldValue *min, const FieldValue *max) {}

    void                ref() { _refs++; }
    void                unref() { if (--_refs == 0) delete this; }
    void                ref0() { if (this != NULL) _refs = 0; }

    virtual FieldValue *copy() = 0;

    virtual MyString    getEcmaScriptComment(MyString name, int flags) 
                              const = 0;

    const char         *getEcmaScriptIndent(int  flags);

    virtual bool        supportAnimation(bool x3d) const = 0;
    virtual bool        supportInteraction(void) const { return false; }
    virtual void        makeEmpty(void) {}

    virtual bool        isX3DType() { return false; }
 
private:
    int                 _refs;
};

class MFieldValue : public FieldValue
{
public:
    virtual int         getSFSize() const = 0;
    virtual FieldValue *getSFValue(int index) const = 0;
    virtual void        setSFValue(int index, FieldValue *value) = 0;
    virtual int         getStride() const { return 1; }
    virtual int         write(int filedes, int indent) const;
    virtual void        removeSFValue(int index) = 0;
    virtual void        makeEmpty(void) 
                           { 
                           for (int i = 0; i < getSFSize(); i++)
                               removeSFValue(i);
                           }
    void                getDiff(IntArray *newIndices, IntArray *oldIndices, 
                                MFieldValue *old);
    virtual bool        supportAnimation(bool x3d) const {return false; }
};

extern int indentf(int filedes, int indent);
extern FieldValue *rewriteField(FieldValue *value, 
                                const char *oldBase, const char *newBase);

#endif // _FIELDVALUE_H
