/*
 * Node.h
 *
 * Copyright (C) 1999 Stephen F. White, 2005 J. "MUFTI" Scheurich
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program (see the file "COPYING" for details); if 
 * not, write to the Free Software Foundation, Inc., 675 Mass Ave, 
 * Cambridge, MA 02139, USA.
 */

#ifndef _NODE_H
#define _NODE_H

#ifndef _STDIO_H
#include <stdio.h>
#endif

#ifndef _ARRAY_H
#include "Array.h"
#endif

#ifndef _LIST_H
#include "List.h"
#endif

#ifndef _VEC3F_H
#include "Vec3f.h"
#endif

#ifndef _DUNE_STRING_H
#include "MyString.h"
#endif

#ifndef _FIELD_INDEX_H
#include "FieldIndex.h"
#endif
#ifndef _ELEMENT_H
#include "Element.h"
#endif

#include "Swap.h"

class FieldValue;
class Node;
class Scene;
class Proto;
class NodePROTO;
class Path;
class NodeList;

class RouteSocket {
public:
                RouteSocket() 
                   { 
                   _node = NULL; 
                   _field = -1; 
                   }
                RouteSocket(Node *node, int index) 
                   { 
                   _node = node; 
                   _field = index; 
                   }

    int         operator==(const RouteSocket &t)
                   { return t._node == _node && t._field == _field; }
 
    int         operator!=(const RouteSocket &t)
                   { return t._node != _node || t._field != _field; }

    Node       *getNode(void) const { return _node; }
    int         getField(void) const { return _field; }
protected:
    Node       *_node;
    int         _field;
};

typedef List<RouteSocket> SocketList;

enum parentFlag {
    PARENT_NOT_TOUCHED,
    PARENT_TOUCHED,
    PARENT_RECURSIVE
};

class Parent {
public:
                Parent(Node *node, int field, Node *self) 
                   { 
                   _node = node; 
                   _field = field;
                   _self = self; 
                   _parentFlag = PARENT_NOT_TOUCHED;
                   }

    int         operator==(const Parent &t)
                   { return t._self == _self; }

    int         operator!=(const Parent &t)
                   { return t._self != _self; }

    Node       *_node;
    int         _field;
    Node       *_self;
    parentFlag  _parentFlag;             
};

typedef List<Parent> ParentList;
typedef ParentList::Iterator *ParentIter;

enum flags {
    NODE_FLAG_SELECTED,
    NODE_FLAG_TOUCHED,
    NODE_FLAG_COLLAPSED,
    NODE_FLAG_DEFED
};

enum Constraint {
    CONSTRAIN_NONE = 0,
    CONSTRAIN_X,
    CONSTRAIN_Y,
    CONSTRAIN_Z,
    CONSTRAIN_XY,
    CONSTRAIN_YZ,
    CONSTRAIN_ZX,
    CONSTRAIN_SPHERE
};



enum NodeEnum {
    DUNE_ANCHOR,
    DUNE_APPEARANCE,
    DUNE_AR_SENSOR,
    DUNE_AUDIO_CLIP,
    DUNE_BACKGROUND,
    DUNE_BALL_JOINT,
    DUNE_BILLBOARD,
    DUNE_BOX,
    DUNE_CATT_EXPORT_REC, // Export data container node for CATT 8
    DUNE_CATT_EXPORT_SRC, // Export data container node for CATT 8
    DUNE_COLLIDABLE_OFFSET,
    DUNE_COLLIDABLE_SHAPE,
    DUNE_COLLISION,
    DUNE_COLLISION_COLLECTION,
    DUNE_COLLISION_SENSOR,
    DUNE_COLLISION_SPACE,
    DUNE_COLOR,
    DUNE_COLOR_INTERPOLATOR,
    DUNE_COLOR_RGBA,
    DUNE_CONE,
    DUNE_CONTACT,
    DUNE_CONTOUR_2D,
    DUNE_COORDINATE,
    DUNE_COORDINATE_DEFORMER,
    DUNE_COORDINATE_INTERPOLATOR,
    DUNE_COORDINATE2D_INTERPOLATOR,
    DUNE_COVER,
    DUNE_CUBE_TEXTURE,
    DUNE_CYLINDER,
    DUNE_CYLINDER_SENSOR,
    DUNE_DIRECTIONAL_LIGHT,
    DUNE_DOUBLE_AXIS_HINGE_JOINT,
    DUNE_ELEVATION_GRID,
    DUNE_EXTRUSION,
    DUNE_FILL_PROPERTIES,
    DUNE_FOG,
    DUNE_FONT_STYLE,
    DUNE_GEO_COORDINATE,
    DUNE_GEO_ELEVATION_GRID,
    DUNE_GEO_LOCATION,
    DUNE_GEO_LOD,
    DUNE_GEO_METADATA,
    DUNE_GEO_ORIGIN,
    DUNE_GEO_POSITION_INTERPOLATOR,
    DUNE_GEO_TOUCH_SENSOR,
    DUNE_GEO_VIEWPOINT,
    DUNE_GROUP,
    DUNE_IMAGE_TEXTURE,
    DUNE_INDEXED_FACE_SET,
    DUNE_INDEXED_LINE_SET,
    DUNE_INLINE,
    DUNE_INLINE_LOAD_CONTROL,
    DUNE_JOYSTICK_SENSOR,
    DUNE_KEY_SENSOR,
    DUNE_LINE_PROPERTIES,
    DUNE_LOAD_SENSOR,
    DUNE_LOD,
    DUNE_MATERIAL,
    DUNE_METADATA_DOUBLE,
    DUNE_METADATA_FLOAT,
    DUNE_METADATA_INTEGER,
    DUNE_METADATA_SET,
    DUNE_METADATA_STRING,
    DUNE_MOTOR_JOINT,
    DUNE_MOVIE_TEXTURE,
    DUNE_MULTI_TEXTURE,
    DUNE_NAVIGATION_INFO,
    DUNE_NORMAL,
    DUNE_NORMAL_INTERPOLATOR,
    DUNE_NURBS_CURVE,
    DUNE_NURBS_CURVE_2D,
    DUNE_NURBS_GROUP,
    DUNE_NURBS_POSITION_INTERPOLATOR,
    DUNE_NURBS_SET,
    DUNE_NURBS_SURFACE,
    DUNE_NURBS_TEXTURE_SURFACE,
    DUNE_ORIENTATION_INTERPOLATOR,
    DUNE_PIXEL_TEXTURE,
    DUNE_PLANE_SENSOR,
    DUNE_POINT_LIGHT,
    DUNE_POINT_SET,
    DUNE_POLYLINE_2D,
    DUNE_POSITION_INTERPOLATOR,
    DUNE_POSITION2D_INTERPOLATOR,
    DUNE_PROXIMITY_SENSOR,
    DUNE_RIGID_BODY,
    DUNE_RIGID_BODY_COLLECTION,
    DUNE_SCALAR_INTERPOLATOR,
    DUNE_SCRIPT,
    DUNE_SHAPE,
    DUNE_SINGLE_AXIS_HINGE_JOINT,
    DUNE_SKY,
    DUNE_SLIDER_JOINT,
    DUNE_SOUND,
    DUNE_SPACE_SENSOR,
    DUNE_SPHERE,
    DUNE_SPHERE_SENSOR,
    DUNE_SPOT_LIGHT,
    DUNE_STRING_SENSOR,
    DUNE_STEERING_WHEEL,
    DUNE_SUPER_ELLIPSOID,
    DUNE_SUPER_EXTRUSION,
    DUNE_SUPER_SHAPE,
    DUNE_SWITCH,
    DUNE_TEXT,
    DUNE_TEXTURE_COORDINATE,
    DUNE_TEXTURE_TRANSFORM,
    DUNE_TIME_SENSOR,
    DUNE_TOUCH_SENSOR,
    DUNE_TRANSFORM,
    DUNE_TRIANGLE_SET,
    DUNE_TRIMMED_SURFACE,
    DUNE_TUI_BUTTON,
    DUNE_TUI_COMBO_BOX,
    DUNE_TUI_FLOAT_SLIDER,
    DUNE_TUI_FRAME,
    DUNE_TUI_LABEL,
    DUNE_TUI_LIST_BOX,
    DUNE_TUI_MAP,
    DUNE_TUI_PROGRESS_BAR,
    DUNE_TUI_SLIDER,
    DUNE_TUI_SPLITTER,
    DUNE_TUI_TAB,
    DUNE_TUI_TAB_FOLDER,
    DUNE_TUI_TOGGLE_BUTTON,
    DUNE_UNIVERSAL_JOINT,
    DUNE_VEHICLE,
    DUNE_VIEWPOINT,
    DUNE_VISIBILITY_SENSOR,
    DUNE_VRML_CUT,
    DUNE_VRML_SCENE,
    DUNE_WAVE,
    DUNE_WORLD_INFO,
    DUNE_COMMENT
};

// the following are node "classes", used to validate the
// scene graph hierarchy
enum {
    ANY_NODE                       = 1<<11,
    CHILD_NODE                     = 1<<12, 
    PROTO_NODE                     = 1<<14,
    GEOMETRY_NODE                  = 1<<15,
    PARAMETRIC_GEOMETRY_NODE       = 1<<16,
    COLOR_NODE                     = 1<<17,
    METADATA_NODE                  = 1<<18,
    EFFECT_NODE                    = 1<<19,
    NURBS_CONTROL_CURVE_NODE       = 1<<20,
    NURBS_TEXTURE_SURFACE_NODE     = 1<<21,
    TEXTURE_COORDINATE_NODE        = 1<<22,
    TEXTURE_NODE                   = 1<<23,
    SOUND_SOURCE_NODE              = 1<<24,
    URL_NODE                       = 1<<25,
    INTERPOLATOR_NODE              = 1<<26,
    X3DN_BODY_COLLIDABLE_NODE      = 1<<27,
    X3DN_BODY_COLLISION_SPACE_NODE = 1<<28,
    X3D_RIGID_JOINT_NODE           = 1<<29,
    X3D_MASS_DENSITY_MODEL_NODE    = 1<<30
    // do not use sign bit 31
};

enum {
    PROFILE_CORE,
    PROFILE_INTERCHANGE,
    PROFILE_INTERACTIVE,
    PROFILE_MPEG4_INTERACTIVE,
    PROFILE_IMMERSIVE,
    PROFILE_FULL
};

class NodeData {
public:
                        NodeData(Scene *scene, Proto *proto);
                        NodeData(const Node &node);
    virtual            ~NodeData();

    void                delete_this(void);
    bool                isClassType(int type) { return type >= ANY_NODE; }

    virtual int         write(int filedes, int indent);
    virtual int         writeProto(int filedes);
    int                 writeProto(int f, const char *urn, 
                                         const char *directoryName,
                                         const char *haveUrl = "");
    int                 writeProtoArguments(int filedes) const;
    virtual bool        canWriteAc3d();
    virtual int         writeAc3d(int filedes, int indent);
    virtual int         writeAc3dMaterial(int &materialIndex, int filedes, 
                                          int indent);
    Scene              *getScene() const { return _scene; }
    virtual FieldValue *getField(int index) const;
    virtual void        setField(int index, FieldValue *value);

    virtual FieldValue *getField(FieldIndex index) const
                           { return getField((int) index); }
    virtual void        setField(FieldIndex index, FieldValue *value)
                           { setField((int) index, value); }

    const char         *getTreeLabel(void) { return _treeLabel; }
    const MyString     &getName(bool newName = false);
    // setName should only be called from Scene::def
    void                setName(const char *name);
    bool                hasName(void);
    bool                needsDEF() const;

    void                getGraphPosition(float *x, float *y) const
                           { *x = _graphX; *y = _graphY; }
    void                setGraphPosition(float x, float y)
                           { _graphX = x; _graphY = y; }

    void                getGraphSize(int *width, int *height) const
                           { *width = _graphWidth; *height = _graphHeight; }
    void                setGraphSize(int width, int height)
                           { _graphWidth = width; _graphHeight = height; }

    void                ref() { _refs++; }
    void                unref() 
                           { 
                           if (--_refs == 0) 
                               delete_this(); 
                           }
    void                ref0() { if (this != NULL) _refs = 0; }
    int                 getRefs() { return _refs; }

    virtual void        onAddRoute(Node *s, int out, Node *d, int in) {}

    virtual int         getType() const = 0;
    virtual int         getNodeClass() const { return CHILD_NODE; }

    virtual int         getProfile(void) const { return PROFILE_IMMERSIVE; }

    virtual const char* getComponentName(void) const { return ""; }
    virtual int         getComponentLevel(void) const { return -1; }

    virtual Node       *copy() const = 0;
                           
    int                 findChild(Node *child, int field) const;

    int                 lookupEventIn(const MyString &name, bool x3d) const;
    int                 lookupEventOut(const MyString &name, bool x3d) const;

    Proto              *getProto() const { return _proto; }
    virtual Proto      *getPrimaryProto() const { return getProto(); }
 
    void                addInput(int eventIn, Node *src, int eventOut);
    void                addOutput(int eventOut, Node *dst, int eventIn);
    void                removeInput(int eventIn, Node *src, int eventOut);
    void                removeOutput(int eventOut, Node *dst, int eventIn);
    const SocketList   &getInput(int i) const { return _inputs[i]; }
    const SocketList   &getOutput(int i) const { return _outputs[i]; }

    virtual void        update();
    virtual void        reInit();

    Node               *getParent(ParentIter iter) const 
                              { return iter->item()._node; }
    int                 getParentField(ParentIter iter) const 
                              { return iter->item()._field; }

    bool                checkValidX3d(Element *field);
    bool                validChild(int field, Node *child);
    bool                validChildType(int field, int childType);
    int                 findValidField(Node *child);
    int                 findValidFieldType(int childType);
    int                 findFirstValidFieldType(int childType);
    Path               *getPath() const;

    bool                getFlag(int flag) const { return (_flags & (1 << flag)) != 0; }
    void                setFlag(int flag) { _flags |= (1 << flag); }
    void                setFlagRec(int flag);
    void                clearFlag(int flag) { _flags &= ~(1 << flag); }
    void                clearFlagRec(int flag);

    bool                isSelected() const { return getFlag(NODE_FLAG_SELECTED); }
    bool                isCollapsed() const { return getFlag(NODE_FLAG_COLLAPSED); }

    virtual void        transform() {}
    virtual void        transformBranch() {}
    virtual void        transformForHandle(int /* handle */) {}
    virtual void        preDraw() {}
    virtual void        draw() {}
    virtual void        draw(int pass) {}
    virtual void        bind() {}
    virtual void        unbind() {}
    virtual void        drawHandles() {}
    /// countPolygons() do not count polygons in primitives
    virtual int         countPolygons(void) {return 0;}
    virtual int         countPrimitives(void) {return 0;}
    virtual int         countPolygons1Sided(void) {return 0;}
    virtual int         countPolygons2Sided(void) {return 0;}

    virtual bool        isMesh(void) { return _isMesh; }

    virtual bool        hasTwoSides(void) { return false; }
    virtual bool        isDoubleSided(void) { return false; }
    virtual void        toggleDoubleSided(void) {}
    virtual int         getSolidField() { return -1; }
    virtual void        flipSide(void) {}

    virtual Vec3f       getHandle(int handle, int *constraint, int *field)
                           { 
                           *field = -1;
                           return Vec3f(0.0f, 0.0f, 0.0f); 
                           }
    virtual void        setHandle(int /* handle */, const Vec3f & /* v */) {}

    virtual void        sendEvent(int eventOut, double timestamp, FieldValue *value);
    virtual void        receiveEvent(int eventIn, double timestamp, FieldValue *value);

    virtual bool        isInterpolator() { return false; }

    //nurbs conversion virtual
    virtual Node        *toNurbs() {return NULL;}
    virtual Node        *toNurbs(int nshell, int narea, int narcs, 
                                 int uDegree, int vDegree) {return NULL;}
    virtual Node        *toNurbs(int narcslong,  int narcslat, 
                                 int uDegree, int vDegree) {return NULL;}
    virtual Node        *toNurbs(int nAreaPoints, int Degree) {return NULL;}
    virtual Node        *toNurbs(int narcs, int pDegree, float rDegree, 
                                 int axis) {return NULL;}
    virtual Node        *toNurbs(int narcs, int pDegree, float uDegree, 
                                 Vec3f &P1, Vec3f &P2) {return NULL;}
    virtual Node        *degreeElevate(int newDegree) {return NULL;}
    virtual Node        *degreeElevate(int newUDegree, int newVDegree)
                            {return NULL;}

    // mesh conversion virtual
    virtual Node        *toIndexedFaceSet(bool wantNormal = true,
                                          bool cleanVertices = true) 
                            { return NULL; }
    virtual bool         canConvertToIndexedFaceSet(void) { return false; }

    // triangle set conversion virtual
    virtual Node        *toTriangleSet(void) { return NULL; }
    virtual bool         canConvertToTriangleSet(void) { return false; }

    // extrusion conversion virtual
    virtual Node        *toExtrusion(void) { return NULL; }
    virtual bool         canConvertToExtrusion(void) { return false; }

    /// compare content
    bool                isEqual(Node* Node);
    friend bool         isEqual(Node* Node);

    MyString            newEventName(int typeEnum, bool out);

    bool                hasFieldFlag(int flag);
    virtual bool        isInvalidChildNode(void)
                           {
                           // true if node may not be part of a MFNode field
                           // of a other node (or at root of scenegraph)
                           return false;
                           }

    virtual bool        isCoverNode(void) { return false; }
    virtual bool        hasCoverFields(void) { return false; }
    bool                hasCoverDefault() { return hasDefault(FF_COVER_ONLY); }
    virtual bool        isKambiNode(void) { return false; }
    virtual bool        hasKambiFields(void) { return false; }
    bool                hasKambiDefault() { return hasDefault(FF_KAMBI_ONLY); }
    bool                hasX3dDefault() { return hasDefault(FF_X3D_ONLY); }
    bool                hasDefault(int flag);

    virtual void        setNormalFromMesh(Node *nnormal) {}
    virtual void        setTexCoordFromMesh(Node *nnormal) {}

    virtual bool        isTransparent(void) { return false; }
    virtual float       getTransparency(void) { return 0; }

    void                handleIs(void);
    Node               *getIsNode(int nodeIndex);
    void                setScene(Scene *scene) { _scene = scene; }
    bool                isInsideProto(void) { return _inSideProto; }
    void                setInsideProto(bool f = true) { _inSideProto = f; }
    void                setNodePROTO(NodePROTO *node) 
                           { 
                           _nodePROTO = node; 
                           _inSideProto = true;
                           }
    NodePROTO          *getNodePROTO(void) { return _nodePROTO; }
    void                removeChildren(void);
    bool                isDefault(int field) const;
    bool                hasInputs(void);
    bool                hasOutputs(void);
    bool                hasInput(const char* routename) const;
    bool                hasOutput(const char* routename) const;
    int                 writeRoutes(int f, int indent) const;

    void                generateTreeLabel(void);

    virtual void        flatten(int direction, bool zero) {}
    virtual bool        canFlatten(void) {return false;}

    virtual Node       *convert2X3d(void);
    virtual Node       *convert2Vrml(void);

    virtual void        setupFinished(void) {}

protected:
    int                 writeIs(int f, int indent, 
                                const char *name, const char * isName);
    int                 writeFields(int f, int indent);
    int                 writeField(int f, int indent, int i, 
                                   bool script = false);
    int                 writeEventIn(int f, int indent, int i, 
                                     bool eventName = false);
    int                 writeEventOut(int f, int indent, int i, 
                                      bool eventName = false);

protected:
    Scene              *_scene;
    ParentList          _parents;
    MyString            _name;
    MyString            _treeLabel;
    int                 _refs;
    int                 _numberUse;
    int                 _flags;
    Proto              *_proto;
    FieldValue        **_fields;
    int                 _numFields;
    int                 _numEventIns;
    int                 _numEventOuts;
    SocketList         *_inputs;
    SocketList         *_outputs;
    float               _graphX, _graphY;
    int                 _graphWidth, _graphHeight;
    unsigned long       _identifier;
    bool                _inSideProto;
    NodePROTO          *_nodePROTO;
    bool                _isMesh;
};
   

typedef bool (*DoWithNodeCallback)(Node *node, void *data);
typedef bool (*DoWithSimilarBranchCallback)(Node *node, Node *similarNode, 
                                            void *data);
typedef void (*DoWithAllElementsCallback)(Element *element, void *data);

class Node : public NodeData
{
public:
                        Node(Scene *scene, Proto *proto);
                        Node(const Node &node);
                        Node(const Node &node, Proto *proto);

    // macros not needed: fieldMacros(SFNode, metadata, Proto)

    void                addParent(Node *parent, int field);
    bool                hasParent(void) const
                           { 
                           if (getNumParents() <= 0) 
                               return false;
                           return (_geometricParentIter != NULL);
                           }
    Node               *getParent(void) const 
                           { 
                           return NodeData::getParent(_geometricParentIter);
                           }
    int                 getParentField(void) const 
                           { 
                           return 
                                NodeData::getParentField(_geometricParentIter);
                           }
    Node               *getParent(int nodeType) const; 
    int                 getParentIndex(void) const;
    FieldValue         *getParentFieldValue(void) const;

    bool                isInScene(Scene* scene) const;
    virtual void        addFieldNodeList(int index, NodeList *value);
    void                removeParent(void);
    bool                hasAncestor(Node *node) const;
    bool                supportAnimation(void);
    bool                supportInteraction(void);
    virtual int         getAnmationCommentID(void) { return -1; }
    virtual int         getInteractionCommentID(void) { return -1; }
    virtual bool        isInvalidChild(void);
    virtual bool        hasBoundingBox(void) { return false; }
    virtual Vec3f       getMinBoundingBox(void);
    virtual Vec3f       getMaxBoundingBox(void);
    virtual void        flip(int index) {}
    virtual void        swap(int fromTo) {}
    virtual bool        showFields() { return false; }
    virtual bool        isPROTO() { return false; }

    void                appendTo(NodeList* nodelist);
    void                appendComment(Node *node);

    void                doWithParents(DoWithNodeCallback callback, void *data,
                                      bool searchInRest = true, 
                                      bool callSelf = true);
    void                doWithSiblings(DoWithNodeCallback callback, void *data,
                                       bool searchInRest = true, 
                                       bool callSelf = true);

    bool                doWithBranch(DoWithNodeCallback callback, void *data,
                                     bool searchInRest = true, 
                                     bool callSelf = true);

    bool                doWithSimilarBranch(DoWithSimilarBranchCallback 
                                            callback, Node *similarNode, 
                                            void *data);

    void                doWithAllElements(DoWithAllElementsCallback callback, 
                                          void *data);

    int                 getNumParents() const { return _parents.size(); }

    parentFlag          getParentFlag(int index) const 
                              { 
                              return _parents.get(index)->item()._parentFlag; 
                              }

    void                setParentFlag(int index, parentFlag flag) 
                              { 
                              Parent parent = _parents.get(index)->item();
                              parent._parentFlag = flag; 
                              _parents.get(index)->setItem(parent);
                              }

    int                 getNumberUse() { return _parents.size(); }

    bool                isFirstUSE(void)
                           { 
                           if (isPROTO())
                              return _geometricParentIter == 
                                     _parents.first()->next(); 
                           else
                              return _geometricParentIter == _parents.first(); 
                           }

    void                copyChildrenTo(Node *copyedNode);

    int                 getPrevSiblingIndex(void);
    int                 getNextSiblingIndex(void);

    virtual bool        isJoint(void) { return false; }
    virtual bool        isHumanoid(void) { return false; }
    virtual bool        canWriteCattGeo();
    virtual int         writeCattGeo(int filedes, int indent);
    ParentIter          getGeometricParentIter(void)
                           { return _geometricParentIter; }

protected:
    virtual            ~Node();

protected:
    ParentIter          _geometricParentIter;
    NodeList           *_commentsList;
};

bool                    hasRoute(SocketList socketlist);

#endif // _NODE_H
