/*
 * NodeBallJoint.h
 *
 * Copyright (C) 1999 Stephen F. White, 2007 J. "MUFTI" Scheurich
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program (see the file "COPYING" for details); if 
 * not, write to the Free Software Foundation, Inc., 675 Mass Ave, 
 * Cambridge, MA 02139, USA.
 */

#ifndef _NODE_BALL_JOINT_H
#define _NODE_BALL_JOINT_H

#include "X3DRigidJointNode.h"

#include "SFMFTypes.h"

#include "ode_include.h"

class ProtoBallJoint : public X3DRigidJointProto {
public:
                    ProtoBallJoint(Scene *scene);

    virtual Node   *create(Scene *scene);

    FieldIndex      anchorPoint;
    FieldIndex      body1AnchorPoint;
    FieldIndex      body2AnchorPoint;
};

class NodeBallJoint : public X3DRigidJointNode {
public:
                    NodeBallJoint(Scene *scene, Proto *proto);

    virtual int     getType() const { return DUNE_BALL_JOINT; }
    virtual Node   *copy() const { return new NodeBallJoint(*this); }

    fieldMacros(SFVec3f, anchorPoint,      ProtoBallJoint)
    fieldMacros(SFVec3f, body1AnchorPoint, ProtoBallJoint)
    fieldMacros(SFVec3f, body2AnchorPoint, ProtoBallJoint)

#ifdef HAVE_LIBODE
    virtual void    updateRequestedOutputs(void);
    virtual void    deleteFromODE(void);
    void            setODEWorld(dWorld wld, dJointGroup grp);
#endif
};

#endif 
