/*
 * NodeColor.cpp
 *
 * Copyright (C) 1999 Stephen F. White
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program (see the file "COPYING" for details); if 
 * not, write to the Free Software Foundation, Inc., 675 Mass Ave, 
 * Cambridge, MA 02139, USA.
 */

#include <stdio.h>
#include "stdafx.h"

#include "NodeColor.h"
#include "Proto.h"
#include "FieldValue.h"
#include "MFColor.h"
#include "NodeShape.h"
#include "NodeAppearance.h"
#include "NodeMaterial.h"

ProtoColor::ProtoColor(Scene *scene)
  : Proto(scene, "Color")
{
    color.set(
          addExposedField(MFCOLOR, "color", new MFColor()));
}

Node *
ProtoColor::create(Scene *scene)
{
    return new NodeColor(scene, this); 
}

NodeColor::NodeColor(Scene *scene, Proto *def)
  : Node(scene, def)
{
}

void
NodeColor::setField(int index, FieldValue *value)
{
    Node::setField(index, value);
    if (hasParent())
        getParent()->update();
}

int 
NodeColor::writeAc3dMaterial(int &materialIndex, int f, int indent)
{
    _ac3dMaterialIndex.resize(0);

    // search for Material node for missing field values like emissiveColor, etc
    Node *parent = NULL;
    if (hasParent())
        parent = getParent(); // geometry node

    if (hasParent())
        parent = getParent(); // shape node

    NodeShape *nshape = NULL;
    if (parent)
        if (parent->getType() == DUNE_SHAPE)
            nshape = (NodeShape *) parent;

    NodeAppearance *nappearance = NULL;
    if (nshape)
        nappearance = (NodeAppearance *)(nshape->appearance()->getValue());

    NodeMaterial *nmaterial = NULL;
    if (nappearance)
        nmaterial = (NodeMaterial *)nappearance->material()->getValue();

    // VRML default Material settings
    float ambientIntensity = 0.2f;
    Vec3f emissiveColor(0, 0, 0);
    Vec3f specularColor(0, 0, 0);
    int shininess = 0.2f * 128;
    float transparency = 0;
    if (nmaterial) {
        ambientIntensity = nmaterial->ambientIntensity()->getValue();

        emissiveColor[0] = nmaterial->emissiveColor()->getValue(0);
        emissiveColor[1] = nmaterial->emissiveColor()->getValue(1);
        emissiveColor[2] = nmaterial->emissiveColor()->getValue(2);

        specularColor[0] = nmaterial->specularColor()->getValue(0);
        specularColor[1] = nmaterial->specularColor()->getValue(1);
        specularColor[2] = nmaterial->specularColor()->getValue(2);
        
        shininess = nmaterial->shininess()->getValue() * 128.0f;

    }    

    for (int i = 0; i < color()->getSFSize(); i++) {
        RET_ONERROR( mywritef(f, "MATERIAL \"rgb\" ") )
        RET_ONERROR( mywritef(f, "rgb %f %f %f  ", 
                              color()->getValue(i)[0],
                              color()->getValue(i)[1],
                              color()->getValue(i)[2]) )
        RET_ONERROR( mywritef(f, "amb %f %f %f  ",
                              ambientIntensity, 
                              ambientIntensity,
                              ambientIntensity) )
        RET_ONERROR( mywritef(f, "emis %f %f %f  ", 
                              emissiveColor[0],
                              emissiveColor[1],
                              emissiveColor[2]) )
        RET_ONERROR( mywritef(f, "spec %f %f %f  ", 
                              specularColor[0],
                              specularColor[1],
                              specularColor[2]) )
        RET_ONERROR( mywritef(f, "shi %d  ", shininess ) )
        RET_ONERROR( mywritef(f, "trans %f\n", transparency) )
        _ac3dMaterialIndex.append(materialIndex++);
    }
    return 0;
}

int
NodeColor::getAc3dMaterialIndex(int i) 
{ 
    if (i < _ac3dMaterialIndex.size())
        return _ac3dMaterialIndex[i]; 
    return -1;
} 

