/*
 * NodeDoubleAxisHingeJoint.cpp
 *
 * Copyright (C) 1999 Stephen F. White, 2007 J. "MUFTI" Scheurich
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program (see the file "COPYING" for details); if 
 * not, write to the Free Software Foundation, Inc., 675 Mass Ave, 
 * Cambridge, MA 02139, USA.
 */

#include <stdio.h>
#ifndef FLT_MAX
# include <float.h>
#endif
#include "stdafx.h"

#include "NodeDoubleAxisHingeJoint.h"
#include "Proto.h"
#include "Field.h"
#include "FieldValue.h"
#include "MFNode.h"
#include "SFBool.h"
#include "DuneApp.h"
#include "Scene.h"

ProtoDoubleAxisHingeJoint::ProtoDoubleAxisHingeJoint(Scene *scene)
  : X3DRigidJointProto(scene, "DoubleAxisHingeJoint")
{
    anchorPoint.set(addExposedField(SFVEC3F, "anchorPoint", new SFVec3f()));
    axis1.set(addExposedField(SFVEC3F, "axis1", new SFVec3f()));
    axis2.set(addExposedField(SFVEC3F, "axis2", new SFVec3f()));
    desiredAngularVelocity1.set(addExposedField(SFFLOAT, 
                                                "desiredAngularVelocity1",
                                                new SFFloat()));
    desiredAngularVelocity2.set(addExposedField(SFFLOAT, 
                                                "desiredAngularVelocity2",
                                                new SFFloat()));
    maxAngle1.set( 
          addExposedField(SFFLOAT, "maxAngle1", new SFFloat(M_PI),
                          new SFFloat(-M_PI), new SFFloat(M_PI)));
    maxTorque1.set(addExposedField(SFFLOAT, "maxTorque1", new SFFloat()));
    maxTorque2.set(addExposedField(SFFLOAT, "maxTorque2", new SFFloat()));
    minAngle1.set( 
          addExposedField(SFFLOAT, "minAngle1", new SFFloat(-M_PI),
                          new SFFloat(-M_PI), new SFFloat(M_PI)));
    stopBounce1.set(addExposedField(SFFLOAT, "stopBounce1", new SFFloat(0), 
                                    new SFFloat(0), new SFFloat(1)));
    stopConstantForceMix1.set(addExposedField(SFFLOAT, 
                                              "stopConstantForceMix1", 
                                              new SFFloat(0.001), 
                                              new SFFloat(0)));
    stopErrorCorrection1.set(addExposedField(SFFLOAT, 
                                              "stopErrorCorrection1", 
                                              new SFFloat(0.8), 
                                              new SFFloat(0), new SFFloat(1)));
    suspensionErrorCorrection.set(addExposedField(SFFLOAT, 
                                                  "suspensionErrorCorrection", 
                                                  new SFFloat(0.8), 
                                                  new SFFloat(0), 
                                                  new SFFloat(1)));
    suspensionForce.set(addExposedField(SFFLOAT, "suspensionForce", 
                                        new SFFloat(0), new SFFloat(0)));
    addEventOut(SFVEC3F, "body1AnchorPoint");
    addEventOut(SFVEC3F, "body2AnchorPoint");
    addEventOut(SFVEC3F, "body1Axis");
    addEventOut(SFVEC3F, "body2Axis");
    addEventOut(SFFLOAT, "hinge1Angle");
    addEventOut(SFFLOAT, "hinge2Angle");
    addEventOut(SFFLOAT, "hinge1AngleRate");
    addEventOut(SFFLOAT, "hinge2AngleRate");
}

Node *
ProtoDoubleAxisHingeJoint::create(Scene *scene)
{ 
    return new NodeDoubleAxisHingeJoint(scene, this); 
}

NodeDoubleAxisHingeJoint::NodeDoubleAxisHingeJoint(Scene *scene, Proto *def)
  : X3DRigidJointNode(scene, def)
{
}
