/*
 * NodeGeoLocation.h
 *
 * Copyright (C) 1999 Stephen F. White
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program (see the file "COPYING" for details); if 
 * not, write to the Free Software Foundation, Inc., 675 Mass Ave, 
 * Cambridge, MA 02139, USA.
 */

#ifndef _NODE_GEO_LOCATION_H
#define _NODE_GEO_LOCATION_H

#ifndef _GEO_NODE_H
#include "GeoNode.h"
#endif
#ifndef _PROTO_MACROS_H
#include "ProtoMacros.h"
#endif
#ifndef _PROTO_H
#include "Proto.h"
#endif

#include "SFMFTypes.h"

class ProtoGeoLocation : public GeoProto {
public:
                    ProtoGeoLocation(Scene *scene);
    virtual Node   *create(Scene *scene);

    FieldIndex children;
    FieldIndex geoCoords;
    FieldIndex geoCoordsX3D;
    FieldIndex bboxCenter;
    FieldIndex bboxSize;
};

class NodeGeoLocation : public GeoNode {
public:
                    NodeGeoLocation(Scene *scene, Proto *proto);

    virtual int     getType() const { return DUNE_GEO_LOCATION; }
    virtual int     getNodeClass() const { return DUNE_GEO_LOCATION; }
    virtual int     getProfile(void) const { return PROFILE_INTERCHANGE; }
    virtual Node   *copy() const { return new NodeGeoLocation(*this); }

    void            setField(int index, FieldValue *value);
    Node           *convert2Vrml(void);

    fieldMacros(MFNode,   children,     ProtoGeoLocation)
    fieldMacros(SFString, geoCoords,    ProtoGeoLocation)
    fieldMacros(SFVec3d,  geoCoordsX3D, ProtoGeoLocation)
    fieldMacros(SFVec3f,  bboxCenter,   ProtoGeoLocation)
    fieldMacros(SFVec3f,  bboxSize,     ProtoGeoLocation)
};

#endif // _NODE_GEO_LOCATION_H

