/*
 * NodeMotorJoint.cpp
 *
 * Copyright (C) 1999 Stephen F. White, 2007 J. "MUFTI" Scheurich
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program (see the file "COPYING" for details); if 
 * not, write to the Free Software Foundation, Inc., 675 Mass Ave, 
 * Cambridge, MA 02139, USA.
 */

#include <stdio.h>
#ifndef FLT_MAX
# include <float.h>
#endif
#include "stdafx.h"

#include "NodeMotorJoint.h"
#include "Proto.h"
#include "Field.h"
#include "FieldValue.h"
#include "MFNode.h"
#include "SFBool.h"
#include "DuneApp.h"
#include "Scene.h"

ProtoMotorJoint::ProtoMotorJoint(Scene *scene)
  : X3DRigidJointProto(scene, "MotorJoint")
{
    axis1Angle.set(addExposedField(SFFLOAT, "axis1Angle", new SFFloat(0),
                              new SFFloat(-M_PI), new SFFloat(M_PI)));
    axis1Torque.set(addExposedField(SFFLOAT, "axis1Torque", new SFFloat(0)));
    axis2Angle.set(addExposedField(SFFLOAT, "axis2Angle", new SFFloat(0),
                              new SFFloat(-M_PI), new SFFloat(M_PI)));
    axis2Torque.set(addExposedField(SFFLOAT, "axis2Torque", new SFFloat(0)));
    axis3Angle.set(addExposedField(SFFLOAT, "axis3Angle", new SFFloat(0),
                              new SFFloat(-M_PI), new SFFloat(M_PI)));
    axis3Torque.set(addExposedField(SFFLOAT, "axis3Torque", new SFFloat(0)));
    enabledAxes.set(addExposedField(SFINT32, "enabledAxes", new SFInt32(1)));
    motor1Axis.set(addExposedField(SFVEC3F, "motor1Axis", new SFVec3f())); 
    motor2Axis.set(addExposedField(SFVEC3F, "motor2Axis", new SFVec3f())); 
    motor3Axis.set(addExposedField(SFVEC3F, "motor3Axis", new SFVec3f())); 
    stop1Bounce.set(addExposedField(SFFLOAT, "stop1Bounce", new SFFloat(0), 
                                    new SFFloat(0), new SFFloat(1)));
    stop1ErrorCorrection.set(addExposedField(SFFLOAT, "stop1ErrorCorrection", 
                                             new SFFloat(0.8), 
                                             new SFFloat(0), new SFFloat(1)));
    stop2Bounce.set(addExposedField(SFFLOAT, "stop2Bounce", new SFFloat(0), 
                                    new SFFloat(0), new SFFloat(1)));
    stop2ErrorCorrection.set(addExposedField(SFFLOAT, "stop2ErrorCorrection", 
                                             new SFFloat(0.8), 
                                             new SFFloat(0), new SFFloat(1)));
    stop3Bounce.set(addExposedField(SFFLOAT, "stop3Bounce", new SFFloat(0), 
                                    new SFFloat(0), new SFFloat(1)));
    stop3ErrorCorrection.set(addExposedField(SFFLOAT, "stop3ErrorCorrection", 
                                             new SFFloat(0.8), 
                                             new SFFloat(0), new SFFloat(1)));
    autoCalc.set(addField(SFBOOL, "autoCalc", new SFBool(false))); 
    addEventOut(SFFLOAT, "motor1Angle");
    addEventOut(SFFLOAT, "motor1AngleRate");
    addEventOut(SFFLOAT, "motor2Angle");
    addEventOut(SFFLOAT, "motor2AngleRate");
    addEventOut(SFFLOAT, "motor3Angle");
    addEventOut(SFFLOAT, "motor3AngleRate");
}

Node *
ProtoMotorJoint::create(Scene *scene)
{ 
    return new NodeMotorJoint(scene, this); 
}

NodeMotorJoint::NodeMotorJoint(Scene *scene, Proto *def)
  : X3DRigidJointNode(scene, def)
{
}
