/*
 * NodeNavigationInfo.cpp
 *
 * Copyright (C) 1999 Stephen F. White
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program (see the file "COPYING" for details); if 
 * not, write to the Free Software Foundation, Inc., 675 Mass Ave, 
 * Cambridge, MA 02139, USA.
 */

#include <stdio.h>
#include "stdafx.h"

#include "NodeNavigationInfo.h"
#include "Scene.h"
#include "Proto.h"
#include "FieldValue.h"
#include "Field.h"
#include "MFFloat.h"
#include "SFBool.h"
#include "SFFloat.h"
#include "MFString.h"
#include "SFString.h"
#include "Node.h"


ProtoNavigationInfo::ProtoNavigationInfo(Scene *scene)
  : Proto(scene, "NavigationInfo")
{
    addEventIn(SFBOOL, "set_bind");
    if (TheApp->getCoverMode()) {
        // non standard Covise/COVER extensions
        addEventIn(SFBOOL, "set_bindLast", FF_COVER_ONLY);
    }
    float values[] = { 0.25f, 1.6f, 0.75f };
    float *v = new float[3];  memcpy(v, values, 3 * sizeof(float));
    avatarSize.set(
          addExposedField(MFFLOAT, "avatarSize", new MFFloat(v, 3), 
          new SFFloat(0.0f)));
    headlight.set(
          addExposedField(SFBOOL, "headlight", new SFBool(true)));
    speed.set(
          addExposedField(SFFLOAT, "speed", new SFFloat(1.0f), 
                          new SFFloat(0.0f)));
    static const char *transistionTypes[] = { "TELEPORT", "LINEAR", "ANIMATE", 
                                              NULL};
    StringArray *defaulttransistionTypes = new StringArray();
    defaulttransistionTypes->append("LINEAR");
    transistionType.set(
          addExposedField(MFSTRING, "transistionType", 
                          new MFString(defaulttransistionTypes), 0, 
                          transistionTypes));
    setFieldFlags(transistionType, FF_X3D_ONLY);

    static const char *types[] = { "ANY", "WALK", "EXAMINE", "FLY", "NONE", 
                                   NULL};
//X3D
//    static const char *types[] = { "ANY", "WALK", "EXAMINE", "FLY", "NONE", 
//                                   "LOOKAT",NULL};
    StringArray *defaulttypes = new StringArray();
    defaulttypes->append("WALK");
//X3D defaulttypes->append("EXAMINE");
    defaulttypes->append("ANY");
    type.set(
          addExposedField(MFSTRING, "type", new MFString(defaulttypes), 0, 
                          types));
    visibilityLimit.set(
          addExposedField(SFFLOAT, "visibilityLimit", new SFFloat(0.0f), 
                          new SFFloat(0.0f)));
    if (TheApp->getCoverMode()) {
        // non standard Covise/COVER extensions
        scale.set(
          addExposedField(SFFLOAT, "scale", new SFFloat(-1.0f)));
        setFieldFlags(scale, FF_COVER_ONLY);
    }
    addEventOut(SFTIME, "bindTime", FF_X3D_ONLY);
    addEventOut(SFBOOL, "isBound");
}

Node *
ProtoNavigationInfo::create(Scene *scene)
{ 
    return new NodeNavigationInfo(scene, this); 
}

NodeNavigationInfo::NodeNavigationInfo(Scene *scene, Proto *def)
  : Node(scene, def)
{
}

void
NodeNavigationInfo::preDraw()
{
    _scene->setHeadlight(headlight()->getValue());
}

int NodeNavigationInfo::getProfile(void) const
{ 
    if (!isDefault(type_Field()))
        if (type())
            for (int i = 0; i < type()->getSFSize(); i++) {        
                const char* typeStr = type()->getValue(i);
                if (strcmp(typeStr, "ANY") == 0)
                    return PROFILE_INTERACTIVE;
                if (strcmp(typeStr, "FLY") == 0)
                    return PROFILE_INTERACTIVE;
                if (strcmp(typeStr, "EXAMINE") == 0)
                    return PROFILE_INTERACTIVE;
                if (strcmp(typeStr, "LOOKAT") == 0)
                    return PROFILE_INTERACTIVE;
            }
    if (!isDefault(avatarSize_Field()))
        return PROFILE_IMMERSIVE;
    if (!isDefault(speed_Field()))
        return PROFILE_IMMERSIVE;
    if (!isDefault(visibilityLimit_Field()))
        return PROFILE_IMMERSIVE;
    return PROFILE_INTERCHANGE;
}

