/*
 * NodePointLight.h
 *
 * Copyright (C) 1999 Stephen F. White
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program (see the file "COPYING" for details); if 
 * not, write to the Free Software Foundation, Inc., 675 Mass Ave, 
 * Cambridge, MA 02139, USA.
 */

#ifndef _NODE_POINTLIGHT_H
#define _NODE_POINTLIGHT_H

#ifndef _NODE_H
#include "Node.h"
#endif
#ifndef _PROTO_MACROS_H
#include "ProtoMacros.h"
#endif
#ifndef _PROTO_H
#include "Proto.h"
#endif

#include "swt.h"

#include "SFMFTypes.h"
#include "resource.h"

class ProtoPointLight : public Proto {
public:
                    ProtoPointLight(Scene *scene);
    virtual Node   *create(Scene *scene);

    FieldIndex ambientIntensity;
    FieldIndex attenuation;
    FieldIndex color;
    FieldIndex intensity;
    FieldIndex location;
    FieldIndex on;
    FieldIndex radius;
};

class NodePointLight : public Node {
public:
                    NodePointLight(Scene *scene, Proto *proto);

    virtual int     getType() const { return DUNE_POINT_LIGHT; }
    virtual int     getProfile(void) const;
    virtual const char* getComponentName(void) const;
    virtual int     getComponentLevel(void) const;
    virtual Node   *copy() const { return new NodePointLight(*this); }

    virtual void    preDraw();
    virtual void    drawHandles();

    virtual int     getAnmationCommentID(void) 
                       { 
                         return IDS_ANIMATION_HELP_POINTLIGHT + swGetLang(); 
                       }

    virtual Vec3f   getHandle(int handle, int *constraint, int *field);
    virtual void    setHandle(int handle, const Vec3f &v);

    fieldMacros(SFFloat, ambientIntensity, ProtoPointLight)
    fieldMacros(SFVec3f, attenuation,      ProtoPointLight)
    fieldMacros(SFColor, color,            ProtoPointLight)
    fieldMacros(SFFloat, intensity,        ProtoPointLight)
    fieldMacros(SFVec3f, location,         ProtoPointLight)
    fieldMacros(SFBool,  on,               ProtoPointLight)
    fieldMacros(SFFloat, radius,           ProtoPointLight)
protected:
    GLenum          _light;
};

#endif // _NODE_POINTLIGHT_H
