/*
 * NodeSingleAxisHingeJoint.cpp
 *
 * Copyright (C) 1999 Stephen F. White, 2007 J. "MUFTI" Scheurich
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program (see the file "COPYING" for details); if 
 * not, write to the Free Software Foundation, Inc., 675 Mass Ave, 
 * Cambridge, MA 02139, USA.
 */

#include <stdio.h>
#ifndef FLT_MAX
# include <float.h>
#endif
#include "stdafx.h"

#include "NodeSingleAxisHingeJoint.h"
#include "Proto.h"
#include "Field.h"
#include "FieldValue.h"
#include "MFNode.h"
#include "SFBool.h"
#include "DuneApp.h"
#include "Scene.h"

ProtoSingleAxisHingeJoint::ProtoSingleAxisHingeJoint(Scene *scene)
  : X3DRigidJointProto(scene, "SingleAxisHingeJoint")
{
    anchorPoint.set(addExposedField(SFVEC3F, "anchorPoint", new SFVec3f()));
    axis.set(addExposedField(SFVEC3F, "axis", new SFVec3f()));
    maxAngle.set( 
          addExposedField(SFFLOAT, "maxAngle", new SFFloat(M_PI)));
// ?                      new SFFloat(-M_PI), new SFFloat(M_PI)));
    minAngle.set( 
          addExposedField(SFFLOAT, "minAngle", new SFFloat(-M_PI)));
// ?                      new SFFloat(-M_PI), new SFFloat(M_PI)));
    stopBounce.set(addExposedField(SFFLOAT, "stopBounce", new SFFloat(0), 
                                   new SFFloat(0), new SFFloat(1)));
    stopErrorCorrection.set(addExposedField(SFFLOAT, 
                                            "stopErrorCorrection", 
                                            new SFFloat(0.8), 
                                            new SFFloat(0), new SFFloat(1)));
    addEventOut(SFFLOAT, "angle");
    addEventOut(SFFLOAT, "angleRate");
    addEventOut(SFVEC3F, "body1AnchorPoint");
    addEventOut(SFVEC3F, "body2AnchorPoint");
}

Node *
ProtoSingleAxisHingeJoint::create(Scene *scene)
{ 
    return new NodeSingleAxisHingeJoint(scene, this); 
}

NodeSingleAxisHingeJoint::NodeSingleAxisHingeJoint(Scene *scene, Proto *def)
  : X3DRigidJointNode(scene, def)
{
}
