/*
 * NodeUniversalJoint.cpp
 *
 * Copyright (C) 1999 Stephen F. White, 2007 J. "MUFTI" Scheurich
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program (see the file "COPYING" for details); if 
 * not, write to the Free Software Foundation, Inc., 675 Mass Ave, 
 * Cambridge, MA 02139, USA.
 */

#include <stdio.h>
#ifndef FLT_MAX
# include <float.h>
#endif
#include "stdafx.h"

#include "NodeUniversalJoint.h"
#include "Proto.h"
#include "Field.h"
#include "FieldValue.h"
#include "MFNode.h"
#include "SFBool.h"
#include "DuneApp.h"
#include "Scene.h"

ProtoUniversalJoint::ProtoUniversalJoint(Scene *scene)
  : X3DRigidJointProto(scene, "UniversalJoint")
{
    anchorPoint.set(addExposedField(SFVEC3F, "anchorPoint", new SFVec3f()));
    axis1.set(addExposedField(SFVEC3F, "axis1", new SFVec3f()));
    axis2.set(addExposedField(SFVEC3F, "axis2", new SFVec3f()));
    stop1Bounce.set(addExposedField(SFFLOAT, "stop1Bounce", new SFFloat(0), 
                                    new SFFloat(0), new SFFloat(1)));
    stop1ErrorCorrection.set(addExposedField(SFFLOAT, 
                                              "stop1ErrorCorrection", 
                                              new SFFloat(0.8), 
                                              new SFFloat(0), new SFFloat(1)));
    stop2Bounce.set(addExposedField(SFFLOAT, "stop2Bounce", new SFFloat(0), 
                                    new SFFloat(0), new SFFloat(1)));
    stop2ErrorCorrection.set(addExposedField(SFFLOAT, 
                                              "stop2ErrorCorrection", 
                                              new SFFloat(0.8), 
                                              new SFFloat(0), new SFFloat(1)));
    addEventOut(SFVEC3F, "body1AnchorPoint");
    addEventOut(SFVEC3F, "body1Axis");
    addEventOut(SFVEC3F, "body2AnchorPoint");
    addEventOut(SFVEC3F, "body2Axis");
}

Node *
ProtoUniversalJoint::create(Scene *scene)
{ 
    return new NodeUniversalJoint(scene, this); 
}

NodeUniversalJoint::NodeUniversalJoint(Scene *scene, Proto *def)
  : X3DRigidJointNode(scene, def)
{
}
