/*
 * OutputApp.cpp
 *
 * Copyright (C) 2003 J. "MUFTI" Scheurich
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program (see the file "COPYING" for details); if 
 * not, write to the Free Software Foundation, Inc., 675 Mass Ave, 
 * Cambridge, MA 02139, USA.
 */

#include <errno.h>
#include "stdafx.h"
#include <math.h>

#include "DuneApp.h"
#include "resource.h"
#include "OutputApp.h"
#include "PreferencesApp.h"

#ifndef HAVE_X3DV_TO_X3D_COMMAND
# define HAVE_X3DV_TO_X3D_COMMAND ""
#endif

#ifdef HAVE_CHECK_IN_COMMAND
# define CHECK_IN_COMMAND_ENABLED true
#else
# define CHECK_IN_COMMAND_ENABLED false
# define HAVE_CHECK_IN_COMMAND " "
#endif

#define DEFAULT_INDENT 2
#define DEFAULT_FLOAT_DIGITS 6
#define DEFAULT_MATERIAL_NAME_GENERATION MATERIAL_NAME_FULL

OutputApp::OutputApp()
   {
   _revisionControlCheckinCommand = NULL;
   _x3dv2X3dCommand = NULL;
   }

void
OutputApp::OutputSetDefaults()
   {
#ifdef WIN32
    _keepURLs = true;
#else
    _keepURLs = false;
#endif
   _krFormating = false;
   
   _indent = DEFAULT_INDENT;
   _floatDigits = DEFAULT_FLOAT_DIGITS;
   _normalsOnPureVRML97 = false;
   _materialNameGeneration = DEFAULT_MATERIAL_NAME_GENERATION;
   _revisionControlCheckinFlag = CHECK_IN_COMMAND_ENABLED;
   if (_revisionControlCheckinCommand != NULL)
       free(_revisionControlCheckinCommand);
   _revisionControlCheckinCommand = strdup(HAVE_CHECK_IN_COMMAND);
   if (_x3dv2X3dCommand != NULL)
       free(_x3dv2X3dCommand);
   _x3dv2X3dCommand = strdup(HAVE_X3DV_TO_X3D_COMMAND);
   }

void
OutputApp::OutputLoadPreferences()
   {
   assert(TheApp != NULL);

   OutputSetDefaults();

#ifdef WIN32
    _keepURLs = TheApp->GetBoolPreference("KeepURLs", true);    
#else
    _keepURLs = TheApp->GetBoolPreference("KeepURLs", false);    
#endif 
   _indent = TheApp->GetIntPreference("Indent", DEFAULT_INDENT); 
   _floatDigits = TheApp->GetIntPreference("FloatDigits", 
                                           DEFAULT_FLOAT_DIGITS);
   _krFormating = TheApp->GetBoolPreference("KrFormating", false);    
   _includeProtos = TheApp->GetBoolPreference("IncludeProtos", false);    
   _compress = TheApp->GetBoolPreference("Compress", false);    
   _normalsOnPureVRML97 = TheApp->GetBoolPreference("NormalsOnPureVRML97", 
                                                    false);    
   _materialNameGeneration = TheApp->GetIntPreference(
                             "MaterialNameGeneration",
                             DEFAULT_MATERIAL_NAME_GENERATION);
   _cattExportSrcRec = TheApp->GetBoolPreference("CattExportSrcRec", true);    
   _revisionControlCheckinFlag = TheApp->GetBoolPreference("UseRevisionControl",
                                                           CHECK_IN_COMMAND_ENABLED); 
   if (_revisionControlCheckinCommand != NULL)
       free(_revisionControlCheckinCommand);
   _revisionControlCheckinCommand = strdup(TheApp->GetPreference(
                                           "RevisionControlCheckInCommand",
                                           HAVE_CHECK_IN_COMMAND));
   if (_x3dv2X3dCommand != NULL)
       free(_x3dv2X3dCommand);
   _x3dv2X3dCommand = strdup(TheApp->GetPreference("X3dv2X3dCommand",
                                                   HAVE_X3DV_TO_X3D_COMMAND));
   }

void
OutputApp::OutputSavePreferences()
   {
   assert(TheApp != NULL);

   TheApp->SetBoolPreference("KeepURLs", _keepURLs);
   TheApp->SetIntPreference("Indent", _indent);
   TheApp->SetIntPreference("FloatDigits", _floatDigits); 
   TheApp->SetBoolPreference("KrFormating", _krFormating);
   TheApp->SetBoolPreference("IncludeProtos", _includeProtos);
   TheApp->SetBoolPreference("Compress", _compress);
   TheApp->SetBoolPreference("NormalsOnPureVRML97", _normalsOnPureVRML97);
   TheApp->SetIntPreference("MaterialNameGeneration", 
                            _materialNameGeneration);
   TheApp->SetBoolPreference("CattExportSrcRec", _cattExportSrcRec);
   TheApp->SetBoolPreference("CattExportSrcRec", _cattExportSrcRec);
   TheApp->SetBoolPreference("UseRevisionControl", _revisionControlCheckinFlag);
   TheApp->SetPreference("RevisionControlCheckInCommand", 
                         _revisionControlCheckinCommand);
   TheApp->SetPreference("X3dv2X3dCommand", _x3dv2X3dCommand);
   }

bool parseCommandlineArgumentOutput(int & i,int argc, char** argv)
   {
   bool found = true;
   if (strcmp(argv[i],"-krFormating")==0) 
      {
      TheApp->SetkrFormating(true);
      return found;
      }
   else if (strcmp(argv[i],"-indent")==0) 
      {
      int indent;
      if (i++>=argc) return found;
      if (sscanf(argv[i],"%d",&indent)==1)
         TheApp->SetIndent(indent);
      } 
   else if (strcmp(argv[i],"-floatDigits")==0) 
      {
      int floatDigits;
      if (i++>=argc) return found;
      if (sscanf(argv[i],"%d",&floatDigits)==1)
         TheApp->SetFloatDigits(floatDigits);
      } 
   else
      return false;
  return found;
  }


void
OutputApp::SetFloatDigits(int value)
{
   set_number_of_digits(value);
   _floatDigits = value;
}
