/*
 * OutputApp.h
 *
 * Copyright (C) 2003 J. "MUFTI" Scheurich
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program (see the file "COPYING" for details); if 
 * not, write to the Free Software Foundation, Inc., 675 Mass Ave, 
 * Cambridge, MA 02139, USA.
 */

#ifndef _OUTPUT_APP_H
#define _OUTPUT_APP_H

extern bool parseCommandlineArgumentOutput(int & i,int argc, char** argv);

enum {
    MATERIAL_NAME_BEFORE_UNDERSCORE,
    MATERIAL_NAME_AFTER_UNDERSCORE,
    MATERIAL_NAME_FULL
};

class OutputApp {
public:
                        OutputApp();

    void                SetKeepURLs(bool flag) { _keepURLs = flag; }
    bool                GetKeepURLs() const { return _keepURLs; }        

    void                SetIndent(int value) {_indent = value;}
    int                 GetIndent(void) {return _indent;}

    void                SetFloatDigits(int value);
    int                 GetFloatDigits(void) {return _floatDigits;}

    bool                GetIncludeProtos() const { return _includeProtos; }
    void                SetIncludeProtos(bool flag) { _includeProtos = flag; }

    bool                GetkrFormating() const { return _krFormating; }
    void                SetkrFormating(bool flag) { _krFormating = flag; }

    bool                GetCompress() const { return _compress; }
    void                SetCompress(bool flag) { _compress = flag; }

    bool                GetNormalsOnPureVRML97() const 
                           { return _normalsOnPureVRML97; }        
    void                SetNormalsOnPureVRML97(bool flag) 
                           { _normalsOnPureVRML97 = flag; }

    char               *GetX3dv2X3dCommand(void)
                           { return _x3dv2X3dCommand;}
    void                SetX3dv2X3dCommand(char *command)
                           {
                           if (_x3dv2X3dCommand != NULL)
                               free(_x3dv2X3dCommand);
                           _x3dv2X3dCommand = strdup(command);
                           }

    void                OutputSetDefaults();

    void                OutputLoadPreferences();
    void                OutputSavePreferences();

    int                 GetMaterialNameGeneration() const 
                           { return _materialNameGeneration; }
    void                SetMaterialNameGeneration(int method)
                           { _materialNameGeneration = method; }

    bool                IsMaterialNameBeforeUnderscore(void)
                           { return _materialNameGeneration == 
                                    MATERIAL_NAME_BEFORE_UNDERSCORE; }
    bool                IsMaterialNameAfterUnderscore(void)
                           { return _materialNameGeneration == 
                                    MATERIAL_NAME_AFTER_UNDERSCORE; }
    bool                IsMaterialNameFull(void)
                           { return _materialNameGeneration == 
                                    MATERIAL_NAME_FULL; }

    bool                GetCattExportSrcRec(void) { return _cattExportSrcRec; }
    void                SetCattExportSrcRec(bool flag) 
                           { _cattExportSrcRec = flag; }

    bool                GetRevisionControlCheckinFlag(void) 
                           { return _revisionControlCheckinFlag; }
    void                SetRevisionControlCheckinFlag(bool flag) 
                           { _revisionControlCheckinFlag = flag; }

    const char         *GetRevisionControlCheckinCommand(void) 
                           { return _revisionControlCheckinCommand; }
    void                SetRevisionControlCheckinCommand(const char* command) 
                           {
                           if (_revisionControlCheckinCommand != NULL)
                              free(_revisionControlCheckinCommand); 
                           _revisionControlCheckinCommand = strdup(command); 
                           }

private:
    bool                _keepURLs;
    int                 _indent;
    int                 _floatDigits;
    bool                _krFormating;
    bool                _includeProtos;
    bool                _compress;    
    bool                _normalsOnPureVRML97;
    int                 _materialNameGeneration;
    bool                _cattExportSrcRec;
    bool                _revisionControlCheckinFlag;
    char               *_revisionControlCheckinCommand;
    char               *_x3dv2X3dCommand;
};

#endif

