/*
 * RenderState.h
 *
 * Copyright (C) 2000 Stephen F. White
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program (see the file "COPYING" for details); if 
 * not, write to the Free Software Foundation, Inc., 675 Mass Ave, 
 * Cambridge, MA 02139, USA.
 */

#ifndef _RENDER_STATE_H
#define _RENDER_STATE_H

#ifndef _MATRIX_H
# include "Matrix.h"
#endif

#define HANDLE_SIZE 1.25f

enum {
    NO_HANDLE = 0,
    TRANSLATION,
    TRANSLATION_X,
    TRANSLATION_Y,
    TRANSLATION_Z,
    ROTATION,
    ROTATION_X,
    ROTATION_Y,
    ROTATION_Z,
    SCALE_X,
    SCALE_Y,
    SCALE_Z,
    SCALE,
    UNIFORM_SCALE_X,
    UNIFORM_SCALE_Y,
    UNIFORM_SCALE_Z,
    CENTER_X,
    CENTER_Y,
    CENTER_Z,
    CENTER
};

class RenderState {
public:
                        RenderState();

    void                startDrawHandles();
    void                drawHandle(const Vec3f &point);
    void                drawHandle(const Vec3d &point);
    void                setHandleColor(Scene *scene, int handle);
    void                endDrawHandles();

    static void         drawCone(GLUquadricObj *obj, float x, float y, float z);
    static void         drawCone(GLUquadricObj *obj, float x, float y, float z,
                                                     float r, float b, float g);
    static void         drawSphere(GLUquadricObj *obj, 
                                   float x, float y, float z);
    static void         drawSphere(GLUquadricObj *obj, 
                                   float x, float y, float z,
                                   float r, float b, float g);
    static void         drawAntiCone(GLUquadricObj *obj, 
                                     float x, float y, float z);
    static void         drawAntiCone(GLUquadricObj *obj, 
                                     float x, float y, float z,
                                     float r, float b, float g);
    static void         drawCylinder(GLUquadricObj *obj, 
                                     float x, float y, float z);
    static void         drawCylinder(GLUquadricObj *obj, 
                                     float x, float y, float z,
                                     float r, float b, float g);
    static void         drawScaleHandle(float x, float y, float z, 
                                        bool uniform = false);
    static void         drawAxisLines();
    static void         drawTranslationHandles(Node *node, float scale);
    static void         drawRotationHandles(Node* node, float scale = 1.0f);
    static void         drawScaleHandles(Node *node, float scale, 
                                         const float *fscale);
    static void         drawUniformScaleHandles(Node *node, float scale,
                                                const float *fscale);
    static void         drawCenterHandles(Node *node, float scale);
    static void         draw6DHandles(Node *node, float scale);
    static void         draw6DlocalHandles(Node *node, float scale);
    static void         drawRocketHandles(Node *node, float scale);
    static void         drawHoverHandles(Node *node, float scale);

private:
    Vec3f               project(const Vec3f &point);
    Vec3d               project(const Vec3d &point);

private:
    Matrixd             _modelviewMatrix;
    Matrixd             _projectionMatrix;
    double              _viewport[4];
};

#endif
