/*
 * SFString.cpp
 *
 * Copyright (C) 1999 Stephen F. White
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program (see the file "COPYING" for details); if 
 * not, write to the Free Software Foundation, Inc., 675 Mass Ave, 
 * Cambridge, MA 02139, USA.
 */

#include <stdio.h>
#include "stdafx.h"

#include "SFString.h"
#include "DuneApp.h"

SFString::SFString(const MyString &value)
  : _value(value)
{
}

SFString::SFString(const char *str)
  : _value(str)
{  
}

bool        
SFString::readLine(int index, char *line)
{
    _value = "";
    _value += line;
    return true;
}

bool
SFString::equals(const FieldValue *value) const
{
    return value->getType() == SFSTRING &&
      !strcmp(((SFString *) value)->getValue(), (const char *) _value);
}

int SFString::writeData(int f, int indent, bool compactFormat) const
{
    RET_ONERROR( mywritestr(f ,"\"") )
    RET_ONERROR( mywritestr(f ,_value) )
    RET_ONERROR( mywritestr(f ,"\"\n") )
    TheApp->incSelectionLinenumber();
    return(0);
}

MyString
SFString::getEcmaScriptComment(MyString name, int flags) const
{
    const char *indent = ((FieldValue *)this)->getEcmaScriptIndent(flags);
    MyString ret;
    ret = "";
    if (TheApp->GetEcmaScriptAddAllowedValues()) {
        ret += indent;
        ret += "// allowed values:\n";

        ret += indent;
        ret += "   // 'string value' \n";
    }
    if (TheApp->GetEcmaScriptAddBrowserObject()) {
        ret += indent;
        ret += "// related Browser Object functions:\n";
        if (flags != EL_EVENT_IN) {
            ret += indent;
            ret += "   // ";
            ret += name;
            ret += " = Browser.getName();\n";

            ret += indent;
            ret += "   // ";
            ret += name;
            ret += " = Browser.getVersion();\n";

            ret += indent;
            ret += "   // ";
            ret += name;
            ret += " = Browser.getWorldURL();\n";
        }
        if (flags != EL_EVENT_OUT) {
            ret += indent;
            ret += "   // Browser.setDescription(";
            ret += name;
            ret += ");\n";
        }
    }
    if (TheApp->GetEcmaScriptAddExampleUsage()) {
        if (flags != EL_EVENT_IN) {
            ret += indent;
            ret += "// example usage:\n";

            ret += indent;
            ret += "   // ";
            ret += name;
            ret += " = 'hello';\n";

            ret += indent;
            ret += "   // ";
            ret += name;
            ret += " = Browser.getName();\n";
        } 
    }
    return ret;
}
