/*
 * X3DRigidJointNode.h
 *
 * Copyright (C) 1999 Stephen F. White, 2007 J. "MUFTI" Scheurich
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program (see the file "COPYING" for details); if 
 * not, write to the Free Software Foundation, Inc., 675 Mass Ave, 
 * Cambridge, MA 02139, USA.
 */

#ifndef _X3D_RIGID_JOINT_NODE_H
#define _X3D_RIGID_JOINT_NODE_H

#include "RigidBodyPhysicsNode.h"

#include "SFMFTypes.h"

#include "ode_include.h"

class X3DRigidJointProto : public Proto {
public:
               X3DRigidJointProto(Scene *scene, const char *name);
    FieldIndex body1;
    FieldIndex body2;
    FieldIndex mustOutput;
};

class X3DRigidJointNode : public RigidBodyPhysicsNode {
public:
                    X3DRigidJointNode(Scene *scene, Proto *proto);

    virtual int     getNodeClass() const { return CHILD_NODE | 
                                                  X3D_RIGID_JOINT_NODE; }

    virtual const char* getComponentName(void) const 
                           { return "RigidBodyPhysics"; }
    virtual int         getComponentLevel(void) const { return 2; }

    virtual bool        showFields() { return true; }

    fieldMacros(SFNode,   body1,       X3DRigidJointProto)
    fieldMacros(SFNode,   body2,       X3DRigidJointProto)
    fieldMacros(MFString, mustOutput, X3DRigidJointProto)

#ifdef HAVE_LIBODE
   /**
    * Get the number of valid fields that the user has requested updates for.
    *
    * @return a value greater than or equal to zero
    */
    virtual int         numOutputs() { return 0; }

   /**
    * Instruction to the node to fetch the appropriate field values' output
    * from the physics model and update the outputOnly field with the values.
    */
    virtual void        updateRequestedOutputs(void) {}

    virtual void        setBodiesToODEWorld(void);

protected:
    dBallJoint *m_odeJoint;
#endif
};

#endif

